-----------------------------------------------------------------------------------------------
-- Client Lua Script for InFocus
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "GroupLib"
require "Window"
 
-----------------------------------------------------------------------------------------------
-- InFocus Module Definition
-----------------------------------------------------------------------------------------------
local InFocus = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- TODO: Temp value to use in place of buggy tPlayerInfo.nManaMax
local knManaMax = 1000
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function InFocus:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function InFocus:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
		-- TODO: Populate with dependencies
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- InFocus OnLoad
-----------------------------------------------------------------------------------------------
function InFocus:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("InFocus.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- InFocus OnDocLoaded
-----------------------------------------------------------------------------------------------
function InFocus:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "InFocusForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("if", "OnOpen", self)
		Apollo.RegisterSlashCommand("infocus", "OnOpen", self)
		
		Apollo.RegisterEventHandler("Group_Join", "OnGroupJoin", self)
		Apollo.RegisterEventHandler("Group_Left", "OnGroupLeft", self)

		Apollo.RegisterTimerHandler("UpdateTimer", "OnUpdate", self)		
		Apollo.CreateTimer("UpdateTimer", 0.1, true)
		Apollo.StopTimer("UpdateTimer")
		
		-- Do additional Addon initialization here
		self.wndManaBarList = self.wndMain:FindChild("ManaBarList")
		self.tManaBarList = {}
		self.nMaxGroupSize = 0
	end
end

-----------------------------------------------------------------------------------------------
-- InFocus Event Functions
-----------------------------------------------------------------------------------------------
function InFocus:OnOpen()
	self.wndMain:Invoke() -- show the window
	Apollo.StartTimer("UpdateTimer")
end

function InFocus:OnClose()
	Apollo.StopTimer("UpdateTimer")
	self.wndMain:Close() -- hide the window
end

function InFocus:OnGroupJoin()
	if self.wndMain:IsShown() then
		Apollo.StartTimer("UpdateTimer")
	end
end

function InFocus:OnGroupLeft()
	self.nMaxGroupSize = 0
	Apollo.StopTimer("UpdateTimer")
	self:DestroyManaBarList()
end

function InFocus:OnUpdate()
	-- Check if group size has changed
	local nNewGroupSize = GroupLib.GetGroupMaxSize()
	if self.nMaxGroupSize ~= nNewGroupSize then
		self.nMaxGroupSize = nNewGroupSize
		self:PopulateManaBarList()
	end
	local nMemberCount = GroupLib.GetMemberCount()
	local nVisibleBarCount = 0
	-- Loop through party members setting Mana Progress Bar and Character Name
	for i = 1,nMemberCount do
		local wndManaBarItem = self.tManaBarList[i]
		local tPlayerInfo = GroupLib.GetGroupMember(i)
		-- ShowPlayer returns true when player meets criteria (e.g. not full mana)
		if self:ShowPlayer(tPlayerInfo) then
			wndManaBarItem:FindChild("NameText"):SetText(tPlayerInfo.strCharacterName)
			-- TODO: Change knManaMax to tPlayerInfo.nManaMax when GroupLib issue is resolved
			self:SetProgressBar(wndManaBarItem:FindChild("ManaProgressBar"), 0, tPlayerInfo.nMana, knManaMax)
			wndManaBarItem:Show(true)
			nVisibleBarCount = nVisibleBarCount + 1
		else
			-- ShowPlayer returned false, hide player bar
			wndManaBarItem:Show(false)
		end
	end
	-- Loop through remaining progress bars hiding them
	for i = nMemberCount + 1, self.nMaxGroupSize do
		self.tManaBarList[i]:Show(false)
	end
	-- Resize window based on number of items
	local nManaBarListSizeY = 30 + nVisibleBarCount * 20
	local l, t, r, b = self.wndMain:GetAnchorOffsets()
self.wndMain:SetAnchorOffsets(l, t, l+200, t+nManaBarListSizeY)
	self.wndManaBarList:ArrangeChildrenVert()
end

-----------------------------------------------------------------------------------------------
-- InFocus Functions
-----------------------------------------------------------------------------------------------
function InFocus:PopulateManaBarList()
	self:DestroyManaBarList()
	for i = 1, self.nMaxGroupSize do
		local wndManaBarItem = Apollo.LoadForm(self.xmlDoc, "ManaBarItem", self.wndManaBarList, self)
		wndManaBarItem:Show(false)
		self.tManaBarList[i] = wndManaBarItem
	end
end

function InFocus:DestroyManaBarList()
	if self.tManaBarList == nil then
		return
	end
	for i, wnd in ipairs(self.tManaBarList) do
		wnd:Destroy()
	end
	self.tManaBarList = {}
end

function InFocus:ShowPlayer(tPlayerInfo)
	-- no player no update
	if tPlayerInfo == nil then
		return false
	end
	if tPlayerInfo.eClassId == 1 or tPlayerInfo.eClassId == 2 or tPlayerInfo.eClassId == 5 then
		return false
	end
	-- No need to show players with full mana
	-- TODO: Replace with tPlayerInfo.nManaMax when GroupLib issue is resolved
	if tPlayerInfo.nMana >= knManaMax then
		return false
	end
	-- TODO: Uncomment when GroupLib issue is resolved
	-- if tPlayerInfo.nManaMax == 0 then
	-- 	return false
	-- end

	return true
end

function InFocus:SetProgressBar(wndBar, nMin, nProgress, nMax)
	wndBar:SetFloor(nMin)
	wndBar:SetMax(nMax)
	wndBar:SetProgress(nProgress)
end

-----------------------------------------------------------------------------------------------
-- InFocus Instance
-----------------------------------------------------------------------------------------------
local InFocusInst = InFocus:new()
InFocusInst:Init()
